function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var State =
/*#__PURE__*/
function () {
  function State(initialState) {
    this.state = initialState;
    this.subscribers = [];
    this.setState = this.setState.bind(this);
  }

  var _proto = State.prototype;

  _proto.unsubscribe = function unsubscribe(fn) {
    this.subscribers = this.subscribers.filter(function (f) {
      return f !== fn;
    });
  };

  _proto.subscribe = function subscribe(fn) {
    this.subscribers.push(fn);
  };

  _proto.setState = function setState(createState, callback) {
    var _this = this;

    if (createState === void 0) {
      createState = null;
    }

    return Promise.resolve().then(function () {
      var newState;

      if (typeof createState === "function") {
        newState = createState(_extends({}, _this.state));
      } else {
        newState = createState;
      }

      if (newState == null) {
        if (callback) callback();
        return;
      }

      _this.state = Object.assign({}, _this.state, newState);
      return Promise.all(_this.subscribers.map(function (sub) {
        return sub(_this.state);
      })).then(function () {
        if (callback) {
          return callback(_this.state);
        }
      });
    });
  };

  return State;
}();

module.exports = State;
